/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.layout;

import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.bendpointGroupResize;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.moveAndResizeNode;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.moveNode;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.ElementEditPartBase;
import org.fortiss.tooling.base.ui.editpart.FreeConnectorEditPartBase;

/**
 * Base implementation of {@link IDiagramLayoutConfiguration}. It provides the
 * default layout data storage and free bend point movement on a diagram without
 * grid support.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 265F1FDAF910BCBDCF9DDB2416DAE237
 */
public abstract class DiagramLayoutConfigurationBase implements IDiagramLayoutConfiguration {

	/** {@inheritDoc} */
	@Override
	public boolean useGrid() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public void performBendPointLayout(Point p) {
		// no grid => no change
	}

	/** {@inheritDoc} */
	@Override
	public void performElementMoveAndResize(
			ElementEditPartBase<? extends ILayoutedModelElement> editPart,
			ChangeBoundsRequest request) {
		int dx = request.getMoveDelta().x;
		int dy = request.getMoveDelta().y;
		int dw = request.getSizeDelta().width;
		int dh = request.getSizeDelta().height;
		int minWidth = getMinimalElementSize().width;
		int minHeight = getMinimalElementSize().height;
		moveAndResizeNode(editPart.getModel(), dx, dy, dw, dh, minWidth, minHeight);

		// if an element was resized, move the bendpoints of
		// self-transitions/self-channels
		if(dx == 0 && dy == 0) {
			bendpointGroupResize(editPart.getModel(), request.getEditParts(), dw, dh);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void performFreeConnectorMove(
			FreeConnectorEditPartBase<? extends ILayoutedModelElement> connectorEditPart,
			ChangeBoundsRequest request) {
		int dx = request.getMoveDelta().x;
		int dy = request.getMoveDelta().y;
		moveNode(connectorEditPart.getModel(), dx, dy);
	}
}
