/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.objects;

import org.fortiss.tooling.graphicsGL.ui.camera.Camera;

/**
 * View object that wraps another object and can provide pre- and post-processing.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 3D552A0C14316D2493535A85DB4FAFD1
 */
public abstract class WrappedViewObjectBase extends ViewObjectBase {

	/** The wrapped model object. */
	protected final ViewObjectBase wrapped;

	/** Constructor. */
	public WrappedViewObjectBase(ViewObjectBase wrapped) {
		this.wrapped = wrapped;
	}

	/** Called before rendering the wrapped object. */
	protected abstract void preRendering();

	/** Called after rendering the wrapped object. */
	protected abstract void postRendering();

	/** {@inheritDoc} */
	@Override
	public final void renderObject(Camera c) {
		preRendering();
		wrapped.renderObject(c);
		postRendering();
	}
}
