/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.image;

import java.util.EnumMap;

import org.eclipse.jface.resource.CompositeImageDescriptor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;

/**
 * An OverlayImageDescriptor consists of a main icon and several adornments.
 * 
 * @author deissenb
 * @author juergens
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: 713D8C1C016745B31F51CAA612FADEB9
 */
public class OverlayImageDescriptor extends CompositeImageDescriptor {

	/** Base image on which overlays are drawn */
	private final Image base;

	/** Size of image */
	private final Point size = new Point(16, 16);

	/** List of overlay images and positions */
	private final EnumMap<EOverlayPosition, ImageDescriptor> overlays = new EnumMap<EOverlayPosition, ImageDescriptor>(
			EOverlayPosition.class);

	/**
	 * Constructor
	 * 
	 * @param baseImage
	 *            image on which overlays are drawn
	 */
	public OverlayImageDescriptor(Image baseImage) {
		this.base = baseImage;
	}

	/**
	 * Adds an overlay
	 * 
	 * @param overlay
	 *            overlay image
	 * @param position
	 *            position at which overlay is drawn. Possibly existing overlays
	 *            with same positions are overwritten
	 */
	public void addOverlay(ImageDescriptor overlay, EOverlayPosition position) {
		overlays.put(position, overlay);
	}

	/** {@inheritDoc} */
	@Override
	protected void drawCompositeImage(int width, int height) {
		drawImage(base.getImageData(), 0, 0);
		drawOverlays();
	}

	/** Draws all overlays into */
	private void drawOverlays() {
		for (EOverlayPosition position : overlays.keySet()) {
			drawOverlay(position, overlays.get(position));
		}
	}

	/** Draws a single overlay */
	protected void drawOverlay(EOverlayPosition position,
			ImageDescriptor overlay) {

		ImageData overlayData = overlay.getImageData();
		// Use the missing descriptor if it is not there.
		if (overlayData == null) {
			overlayData = ImageDescriptor.getMissingImageDescriptor()
					.getImageData();
		}

		switch (position) {
		case TOP_LEFT:
			drawImage(overlayData, 0, 0);
			break;
		case TOP_RIGHT:
			drawImage(overlayData, size.x - overlayData.width, 0);
			break;
		case BOTTOM_LEFT:
			drawImage(overlayData, 0, size.y - overlayData.height);
			break;
		case BOTTOM_RIGHT:
			drawImage(overlayData, size.x - overlayData.width, size.y
					- overlayData.height);
			break;
		}

	}

	/** {@inheritDoc} */
	@Override
	protected Point getSize() {
		return size;
	}
}